# -*- coding: utf-8 -*-

# schedulerdialog.py
# This file is part of qarte-4
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2016-2018 Vincent Vande Vyvre
# Licence: GPL3
# Home page: https://launchpad.net/qarte
#
# Deferred downloading scheduler dialog

import logging
lgg = logging.getLogger(__name__)

from datetime import datetime, timedelta

from PyQt5.QtCore import QSize, QDate, QTime
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QDialog, QGridLayout, QHBoxLayout, QVBoxLayout,
                             QRadioButton, QButtonGroup, QSpacerItem, QLabel,
                             QPushButton, QLineEdit, QSizePolicy, QComboBox,
                             QDateEdit, QTimeEdit, QTabWidget, QWidget, 
                             QToolBox, QTreeWidget, QAbstractItemView,
                             QTreeWidgetItem, QToolButton)

from data import VERSION_STR
from logeditor import LogEditor


class SchedulerDialog(QDialog):
    def __init__(self, driver, parent=None):
        super().__init__(parent)
        self.resize(539, 578)
        self.driver = driver
        self.setWindowTitle("Deferred Downloading - Qarte-%s" % VERSION_STR)
        self.setSizeGripEnabled(True)
        self.gridLayout_3 = QGridLayout(self)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.artetv_rdo = QRadioButton(self)
        self.artetv_rdo.setObjectName("artetv_rdo")
        self.buttonGroup = QButtonGroup(self)
        self.buttonGroup.setObjectName("buttonGroup")
        self.buttonGroup.addButton(self.artetv_rdo, 1)
        self.horizontalLayout.addWidget(self.artetv_rdo)
        self.arteconcert_rdo = QRadioButton(self)
        self.arteconcert_rdo.setObjectName("arteconcert_rdo")
        self.buttonGroup.addButton(self.arteconcert_rdo, 2)
        self.horizontalLayout.addWidget(self.arteconcert_rdo)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.title_lbl = QLabel(self)
        self.title_lbl.setText("Video title")
        self.title_lbl.setObjectName("title_lbl")
        self.horizontalLayout_2.addWidget(self.title_lbl)
        self.choose_btn = QPushButton(self)
        self.choose_btn.setMaximumSize(QSize(120, 16777215))
        icon = QIcon()
        icon.addPixmap(QPixmap("/medias/magnifier_20x20.png"), QIcon.Normal, QIcon.Off)
        self.choose_btn.setIcon(icon)
        self.choose_btn.setObjectName("choose_btn")
        self.horizontalLayout_2.addWidget(self.choose_btn)
        spacerItem1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.fname_rdo = QRadioButton(self)
        self.fname_rdo.setObjectName("fname_rdo")
        self.buttonGroup_2 = QButtonGroup(self)
        self.buttonGroup_2.setObjectName("buttonGroup_2")
        self.buttonGroup_2.addButton(self.fname_rdo, 1)
        self.gridLayout.addWidget(self.fname_rdo, 0, 0, 1, 1)
        self.fname_led = QLineEdit(self)
        self.fname_led.setObjectName("fname_led")
        self.gridLayout.addWidget(self.fname_led, 0, 1, 1, 1)
        self.serial_rdo = QRadioButton(self)
        self.serial_rdo.setObjectName("serial_rdo")
        self.buttonGroup_2.addButton(self.serial_rdo, 2)
        self.gridLayout.addWidget(self.serial_rdo, 1, 0, 1, 1)
        self.serials_cmb = QComboBox(self)
        self.serials_cmb.setObjectName("serial_cmb")
        self.serials_cmb.setEnabled(False)
        self.gridLayout.addWidget(self.serials_cmb, 1, 1, 1, 1)
        spacerItem2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 1, 2, 1, 1)
        self.kwords_rdo = QRadioButton(self)
        self.kwords_rdo.setObjectName("kwords_rdo")
        self.buttonGroup_2.addButton(self.kwords_rdo, 3)
        self.gridLayout.addWidget(self.kwords_rdo, 2, 0, 1, 1)
        self.kwords_led = QLineEdit(self)
        self.kwords_led.setObjectName("kwords_led")
        self.kwords_led.setEnabled(False)
        self.gridLayout.addWidget(self.kwords_led, 2, 1, 1, 1)
        spacerItem3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 2, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.last_rdo = QRadioButton(self)
        self.last_rdo.setObjectName("last_rdo")
        self.last_rdo.setEnabled(False)
        self.buttonGroup_3 = QButtonGroup(self)
        self.buttonGroup_3.setObjectName("buttonGroup_3")
        self.buttonGroup_3.addButton(self.last_rdo, 1)
        self.horizontalLayout_3.addWidget(self.last_rdo)
        self.all_rdo = QRadioButton(self)
        self.all_rdo.setObjectName("all_rdo")
        self.all_rdo.setEnabled(False)
        self.buttonGroup_3.addButton(self.all_rdo, 2)
        self.horizontalLayout_3.addWidget(self.all_rdo)
        spacerItem5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.date_rdo = QRadioButton(self)
        self.date_rdo.setObjectName("date_rdo")
        self.buttonGroup_4 = QButtonGroup(self)
        self.buttonGroup_4.setObjectName("buttonGroup_4")
        self.buttonGroup_4.addButton(self.date_rdo, 1)
        self.gridLayout_2.addWidget(self.date_rdo, 0, 0, 1, 1)
        self.date_spb = QDateEdit(self)
        self.date_spb.setCalendarPopup(True)
        self.date_spb.setDisplayFormat("dd-MM-yyyy")
        self.date_spb.setObjectName("date_spb")
        self.gridLayout_2.addWidget(self.date_spb, 0, 1, 1, 1)
        self.time_spb = QTimeEdit(self)
        self.time_spb.setObjectName("time_spb")
        self.gridLayout_2.addWidget(self.time_spb, 0, 2, 1, 1)
        spacerItem6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem6, 0, 3, 1, 1)
        self.date_warn_lbl = QLabel(self)
        self.date_warn_lbl.setText('Warning: time too short!')
        self.date_warn_lbl.setStyleSheet("QLabel {color: red;}")
        self.gridLayout_2.addWidget(self.date_warn_lbl, 1, 0, 1, 2)
        self.week_rdo = QRadioButton(self)
        self.week_rdo.setObjectName("week_rdo")
        self.buttonGroup_4.addButton(self.week_rdo, 2)
        self.gridLayout_2.addWidget(self.week_rdo, 2, 0, 1, 1)
        self.week_cmb = QComboBox(self)
        self.week_cmb.setObjectName("week_cmb")
        self.week_cmb.setEnabled(False)
        self.gridLayout_2.addWidget(self.week_cmb, 2, 1, 1, 1)
        self.time_spb_2 = QTimeEdit(self)
        self.time_spb_2.setObjectName("time_spb_2")
        self.time_spb_2.setEnabled(False)
        self.gridLayout_2.addWidget(self.time_spb_2, 2, 2, 1, 1)
        spacerItem7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem7, 2, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_2)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.quality_lbl = QLabel(self)
        self.quality_lbl.setObjectName("quality_lbl")
        self.horizontalLayout_4.addWidget(self.quality_lbl)
        self.quality_cmb = QComboBox(self)
        self.quality_cmb.setMinimumSize(QSize(200, 0))
        self.quality_cmb.setObjectName("quality_cmb")
        self.horizontalLayout_4.addWidget(self.quality_cmb)
        self.settings_btn = QPushButton(self)
        self.settings_btn.setObjectName("settings_btn")
        self.horizontalLayout_4.addWidget(self.settings_btn)
        spacerItem8 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.create_btn = QPushButton(self)
        self.create_btn.setIcon(icon)
        self.create_btn.setObjectName("create_btn")
        self.create_btn.setEnabled(False)
        self.horizontalLayout_5.addWidget(self.create_btn)
        self.warn_lbl = QLabel(self)
        self.warn_lbl.setText("Warning")
        self.warn_lbl.setObjectName("warn_lbl")
        self.horizontalLayout_5.addWidget(self.warn_lbl)
        self.ok_warn_btn = QPushButton(self)
        self.ok_warn_btn.setObjectName("ok_warn_btn")
        self.horizontalLayout_5.addWidget(self.ok_warn_btn)
        spacerItem9 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem9)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        ########################
        self.toolBox = QToolBox(self)
        self.tasks_pg = QWidget()
        #self.tasks_pg.setGeometry(QRect(0, 0, 665, 240))
        self.verticalLayout_2 = QVBoxLayout(self.tasks_pg)
        self.verticalLayout_2.setContentsMargins(2, 0, 2, 0)
        self.horizontalLayout_6 = QHBoxLayout()
        self.tasks_tree = TasksTree(self, self.tasks_pg)
        self.horizontalLayout_6.addWidget(self.tasks_tree)
        self.verticalLayout_3 = QVBoxLayout()
        self.remove_task_btn = QPushButton(self.tasks_pg)
        self.remove_task_btn.setEnabled(False)
        self.verticalLayout_3.addWidget(self.remove_task_btn)
        spacerItem10 = QSpacerItem(20, 40, QSizePolicy.Minimum, 
                                        QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem10)
        self.horizontalLayout_6.addLayout(self.verticalLayout_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.toolBox.addItem(self.tasks_pg, "")
        self.log_pg = QWidget()
        #self.log_pg.setGeometry(QRect(0, 0, 665, 240))
        self.verticalLayout_4 = QVBoxLayout(self.log_pg)
        self.verticalLayout_4.setContentsMargins(2, 0, 2, 0)
        self.verticalLayout_5 = QVBoxLayout()
        self.horizontalLayout_7 = QHBoxLayout()
        self.copy_btn = QToolButton(self.log_pg)
        icon3 = QIcon()
        icon3.addPixmap(QPixmap("medias/clipboard.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.copy_btn.setIcon(icon3)
        self.copy_btn.setShortcut("Ctrl+C")
        self.horizontalLayout_7.addWidget(self.copy_btn)
        self.select_txt_btn = QToolButton(self.log_pg)
        icon4 = QIcon()
        icon4.addPixmap(QPixmap("medias/select-all.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.select_txt_btn.setIcon(icon4)
        self.select_txt_btn.setShortcut("Ctrl+A")
        self.horizontalLayout_7.addWidget(self.select_txt_btn)
        self.erase_btn = QToolButton(self.log_pg)
        icon5 = QIcon()
        icon5.addPixmap(QPixmap("medias/eraser.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.erase_btn.setIcon(icon5)
        self.erase_btn.setShortcut("Delete")
        self.horizontalLayout_7.addWidget(self.erase_btn)
        self.save_btn = QToolButton(self.log_pg)
        icon6 = QIcon()
        icon6.addPixmap(QPixmap("medias/save.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.save_btn.setIcon(icon6)
        self.save_btn.setShortcut("Ctrl+S")
        self.horizontalLayout_7.addWidget(self.save_btn)
        self.log_file_lbl = QLabel(self.log_pg)
        self.horizontalLayout_7.addWidget(self.log_file_lbl)
        spacerItem11 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem11)
        self.verticalLayout_5.addLayout(self.horizontalLayout_7)
        self.log_editor = LogEditor(self, self.log_pg)
        self.horizontalLayout_10 = QHBoxLayout()
        self.warning_wdg = WarningWidget(self)
        #self.warning_wdg.hide()
        self.horizontalLayout_7.addWidget(self.warning_wdg)
        self.verticalLayout_5.addLayout(self.horizontalLayout_10)
        self.verticalLayout_5.addWidget(self.log_editor)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.toolBox.addItem(self.log_pg, "")
        self.verticalLayout.addWidget(self.toolBox)
        self.gridLayout_3.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi()
        self.toolBox.setCurrentIndex(0)
        self.artetv_rdo.setChecked(True)
        self.fname_rdo.setChecked(True)
        self.last_rdo.setChecked(True)
        self.date_rdo.setChecked(True)
        if driver.lang == "fr":
            from data import SERIALS_FR as serials, DAYS_FR as days
        else:
            from data import SERIALS_DE as serials, DAYS_DE as days
        self.days_names = days
        self.serials_cmb.addItems(serials)
        self.week_cmb.addItems(days)
        self.date_warn_lbl.hide()
        self.settings_btn.hide()
        self.warn_lbl.hide()
        self.ok_warn_btn.hide()
        self.set_current_datetime()

        self.buttonGroup.buttonClicked[int].connect(self.on_site_changed)
        self.buttonGroup_2.buttonClicked[int].connect(self.on_task_changed)
        self.buttonGroup_3.buttonClicked[int].connect(self.on_releases_changed)
        self.buttonGroup_4.buttonClicked[int].connect(self.on_schedule_changed)
        self.choose_btn.clicked.connect(self.driver.set_selection_mode)
        self.date_spb.dateChanged.connect(self.on_date_changed)
        self.time_spb.timeChanged.connect(self.on_date_changed)
        self.settings_btn.clicked.connect(self.driver.call_settings_dialog)
        self.create_btn.clicked.connect(self.driver.create_task)
        self.remove_task_btn.clicked.connect(self.on_delete_clicked)
        self.copy_btn.clicked.connect(self.log_editor.copy_)
        self.select_txt_btn.clicked.connect(self.log_editor.select_all)
        self.erase_btn.clicked.connect(self.log_editor.erase_selected_text)
        self.save_btn.clicked.connect(self.log_editor.save_log)

    def on_site_changed(self, idx):
        lgg.info('Server changed: %s' % idx)
        self.fname_rdo.click()
        self.serial_rdo.setEnabled(idx == 1)
        self.kwords_rdo.setEnabled(idx == 1)
        self.week_rdo.setEnabled(idx == 1)

    def on_task_changed(self, idx):
        lgg.info('Task changed: %s' % idx)
        self.set_releases_enabled(idx in (2, 3))
        self.serials_cmb.setEnabled(idx == 2)
        self.kwords_led.setEnabled(idx == 3)

    def on_releases_changed(self, idx):
        lgg.info('Releases changed: %s' % idx)

    def on_schedule_changed(self, idx):
        lgg.info('Schedule changed: %s' % idx)
        self.date_spb.setEnabled(idx == 1)
        self.time_spb.setEnabled(idx == 1)
        self.week_cmb.setEnabled(idx == 2)
        self.time_spb_2.setEnabled(idx == 2)
        self.create_btn.setEnabled(idx == 2)

    def on_date_changed(self, change):
        d = self.date_spb.date()
        t = self.time_spb.time()
        valid = self.check_datetime(d, t)
        self.set_invalid_date_warning(not valid)
        self.create_btn.setEnabled(valid)

    def on_delete_clicked(self):
        select = self.tasks_tree.get_selection()
        if select:
            lgg.info('Delete: %s' % select[0].text(3))
            self.driver.remove_task(select[0].text(0))
            self.tasks_tree.remove_selection()
            self.remove_task_btn.setEnabled(False)

    def set_releases_enabled(self, b):
        self.last_rdo.setEnabled(b)
        self.all_rdo.setEnabled(b)

    def configure(self, video):
        self.video = video
        lgg.info('Configure video: %s' % video.title[:30])
        try:
            _ = video.category
            self.set_arte_concert_state()
        except AttributeError:
            # arte TV item have no attribute category
            self.arteconcert_rdo.setEnabled(False)
            self.set_fname_state()

        self.choose_btn.hide()
        self.title_lbl.setText(video.title)
        self.fname_led.setText(video.fname)
        self.create_btn.setEnabled(True)

    def set_current_datetime(self):
        val = datetime.now()
        val = val + timedelta(minutes=2)
        date = QDate()
        date.setDate(val.year, val.month, val.day)
        self.date_spb.setMinimumDate(date)
        self.date_spb.setDate(date)
        time = QTime()
        time.setHMS(val.hour, val.minute, 0)
        self.time_spb.setTime(time)

    def check_datetime(self, d, t):
        """Check if date and time of cron are not too short.

        Args:
        d -- QDate
        t -- QTime

        Returns:
        False if datetime is too short, True otherwise
        """
        now = datetime.now()
        dt = datetime(d.year(), d.month(), d.day(), t.hour(), t.minute(), 0)
        if now >= dt:
            return False

        return True

    def set_invalid_date_warning(self, b):
        lgg.info('Invalid datetime')
        self.date_warn_lbl.setVisible(b)
        self.create_btn.setEnabled(not b)

    def set_fname_state(self):
        self.set_releases_enabled(False)
        self.serial_rdo.setEnabled(False)
        self.kwords_rdo.setEnabled(False)
        self.week_rdo.setEnabled(False)
        self.quality_lbl.hide()
        self.quality_cmb.hide()
        self.settings_btn.show()

    def set_arte_concert_state(self):
        self.arteconcert_rdo.setChecked(True)
        self.artetv_rdo.setEnabled(False)
        self.set_fname_state()

    def closeEvent(self, e):
        self.log_editor.stopped = True
        QDialog.closeEvent(self, e)

    def retranslateUi(self):
        self.artetv_rdo.setText(_("arte TV Guide"))
        self.arteconcert_rdo.setText(_("arte Concerts"))
        self.fname_rdo.setText(_("File name"))
        self.choose_btn.setToolTip(_("Choose a video into the main window"))
        self.choose_btn.setText(_("Choose a video"))
        self.serial_rdo.setText(_("Serial"))
        self.kwords_rdo.setText(_("Keywords"))
        self.all_rdo.setText(_("All releases"))
        self.last_rdo.setText(_("Last release"))
        self.date_rdo.setText(_("Date"))
        self.week_rdo.setText(_("Week"))
        self.quality_lbl.setText(_("Quality:"))
        self.settings_btn.setText(_("Settings"))
        self.create_btn.setText(_("Create"))
        self.ok_warn_btn.setText(_("OK"))
        self.remove_task_btn.setToolTip(_("Remove selected tasks"))
        self.remove_task_btn.setText(_("Delete"))
        self.toolBox.setItemText(0, _("Tasks"))
        self.copy_btn.setToolTip(_("Copy selected text into the clipboard Ctrl+C"))
        self.select_txt_btn.setToolTip(_("Select all text Ctrl+A"))
        self.erase_btn.setToolTip(_("Delete selected text Delete"))
        self.save_btn.setToolTip(_("Save"))
        self.toolBox.setItemText(1, _("Log file"))

class TasksTree(QTreeWidget):
    def __init__(self, gui, parent=None):
        super().__init__(parent)
        self.gui = gui
        self.main = gui.driver
        self.setColumnCount(4)
        self.headerItem().setText(0, "ID")
        self.headerItem().setText(1, "Date")
        self.headerItem().setText(2, "Hour")
        self.headerItem().setText(3, "Movie")
        self.setColumnWidth(0, 60)
        self.setColumnWidth(1, 150)
        self.header().setCascadingSectionResizes(True)
        self.setAlternatingRowColors(True)
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.currentItemChanged.connect(self.on_item_changed)
        self.itemClicked.connect(self.on_item_clicked)
        self.editable = False

    def set_single_task(self, idx, item):
        lgg.info('Show task: %s' % idx)
        date = self.get_date(item['date'])
        hour = "%s:%s" %(item['time'][0], item['time'][1])
        wdg = QTreeWidgetItem(self)
        if item['type'] == 2:
            name = '"%s"' % " ".join(item['kwords'])

        else:
            name = item['title']
        self.set_values(wdg, idx, date, hour, name)

    def set_values(self, tw, index, day, time, title):
        tw.setText(0, index)
        tw.setText(1, day)
        tw.setText(2, time)
        tw.setText(3, title)

    def set_editable(self):
        sel = self.get_selection()
        if not sel:
            return False

        self.editable = sel[0]
        return int(sel[0].text(0))

    def update_task(self, item):
        self.set_values(self.editable, item)
        self.clearSelection()
        self.gui.remove_task_btn.setEnabled(False)

    def get_selection(self):
        return self.selectedItems()

    def get_selected_tasks(self):
        sel = self.get_selection()
        if not sel:
            return None

        tasks = []
        for s in sel:
            tasks.append(int(s.text(0)))

        return tasks

    def remove_selection(self):
        lgg.info('Remove selection')
        for item in self.get_selection():
            idx = self.indexOfTopLevelItem(item)
            self.takeTopLevelItem(idx)

    def on_item_changed(self, new, old):
        pass

    def on_item_clicked(self, item, idx):
        self.gui.remove_task_btn.setEnabled(len(self.get_selection()) == 1)

    def get_date(self, value):
        if isinstance(value, int):
            return self.gui.days_names[value]

        return '%s-%s-%s' % (value[0], value[1], value[2])

class WarningWidget(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.layout = QHBoxLayout(self)
        self.label = QLabel(self)
        self.label.setText(_("Log file has been modified on disk!"))
        font = QFont("sans", 10)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setStyleSheet("QLabel {background: #FFBB00};")
        self.layout.addWidget(self.label)
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                        QSizePolicy.Minimum)
        self.layout.addItem(spacer)
        self.reload_btn = QPushButton(self)
        self.reload_btn.setText(_("Reload"))
        self.layout.addWidget(self.reload_btn)
        self.reload_btn.clicked.connect(ui.log_editor.load_log)


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    Dialog = QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())

