# -*- coding: utf-8 -*-

# loader.py
# This file is part of qarte-4
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2011-2018 Vincent Vande Vyvre
# Licence: GPL3
# Home page: https://launchpad.net/qarte
#
# A hacked version of urllib.urlretrieve

import contextlib
import urllib.request
import time

from urllib.error import URLError

from PyQt5.QtCore import QObject, pyqtSignal

class Loader(QObject):
    loadingProgress = pyqtSignal(int, int, int)
    loadingFinished = pyqtSignal(str)
    def __init__(self):
        super().__init__()
        self.is_alive = False

    def config_loader(self, url, filename):
        self.url = url
        self.filename = filename
        self.kill_request = False
        self.last_error = ''
        try:
            with contextlib.closing(urllib.request.urlopen(url, None)) as fp:
                headers = fp.info()
                if "content-length" in headers:
                    self.size = int(headers["Content-Length"])
                else:
                    self.size = 0
        except URLError as why:
            self.size = why

        return self.size

    def download_mp4(self):
        """A hacked version of urllib.request.urlretrieve

        """
        self.is_alive = True
        size = -1
        try:
            with contextlib.closing(urllib.request.urlopen(self.url, None)) as fp:
                bs = 1024*16
                percent = 1
                blocknum = 0
                amount = 0
                blocks_percent = 0
                begin_at = time.time()
                if self.size:
                    blocks_percent = int(self.size / bs / 100)

                with open(self.filename, 'wb') as target:
                    while True:
                        block = fp.read(bs)
                        if not block or self.kill_request:
                            break

                        target.write(block)
                        blocknum += 1
                        amount += bs
                        if blocknum == blocks_percent:
                            elapsed = time.time() - begin_at
                            speed = int(amount / elapsed)
                            remain = int(self.size - amount)
                            self.loadingProgress.emit(percent, speed, remain)
                            percent += 1
                            blocknum = 0
        except URLError as why:
            self.last_error = why
            self.filename = ''

        self.is_alive = False
        self.loadingFinished.emit(self.filename)
